import django.test as dtest
import rest_framework.test as rtest
from django.contrib.auth.models import User
import rest_framework.status as response_status
from lofardata.models import DataLocation


class TestDataLocation(dtest.TestCase):

    def test_insert(self):
        location = DataLocation(name='test', uri='srm://path_to_file/')
        location.save()

        self.assertTrue(location.pk is not None, 'Cannot save object')

    def test_double_insert_unique_entry(self):
        location = DataLocation(name='test', uri='srm://path_to_file/')
        location.save()

        self.assertTrue(location.pk is not None, 'Cannot save object')
        location_first_pk = location.pk

        location = DataLocation(name='test', uri='srm://path_to_file/')
        location.save()

        self.assertTrue(location.pk is not None, 'Cannot save object')
        location_second_pk = location.pk

        self.assertTrue(location_first_pk == location_second_pk, 'Double inserted index do not coincide')

        self.assertTrue(DataLocation.objects.count() == 1)

    def test_insert_from_string(self):
        dataloc = DataLocation.insert_location_from_string('srm://test_site:44321/')

        self.assertTrue(dataloc.pk is not None, 'Cannot save object')
        self.assertEqual('Unknown', dataloc.name)
        self.assertEqual('test_site:44321', dataloc.uri)


class TestRESTAPI(rtest.APITestCase):
    def setUp(self):
        self.user = User.objects.create_superuser('admin')
        self.client.force_authenticate(self.user)

    def test_create_error(self):
        response = self.client.post('/ldvspec/api/v1/data-location/', data=dict(name='testname'),
                                    format='json')
        self.assertEqual(response_status.HTTP_400_BAD_REQUEST, response.status_code)

    def test_create(self):
        response = self.client.post('/ldvspec/api/v1/data-location/', data=dict(name='testname', uri='srm://myniceuri/'),
                                    format='json')
        self.assertEqual(response_status.HTTP_201_CREATED, response.status_code)
        self.assertEqual('srm://myniceuri/', DataLocation.objects.get(name='testname').uri)